#-----------------------------------------------------------------------------------
#Automated Migration to VMxNet3 and Paravirtual Controller for vSphere 4.x
#Created by Dane Young, Entisys Solutions 2011
#Build 2011.5.15 Revision 3
#-----------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------
# The user defined variables below can be set to either $true or $false
# $false uses default settings for new network adapters (All components enabled)
# Changing these variables to $true removes the applicable components
#-----------------------------------------------------------------------------------
$DisableIPv6 = $false # If $true, removes the TCP/IPv6 Component from the New VMxNet3 Adapter
$DisablePacer = $false # If $true, removes the QoS Packet Scheduler from the New VMxNet3 Adapter
$DisableLLTDIO = $false # If $true, removes the Link-Layer Topology Discovery Mapper I/O Driver Component from the New VMxNet3 Adapter
$DisableRSPNDR = $false # If $true, removes the Link-Layer Topology Discovery Responder Component from the New VMxNet3 Adapter
$EnableNetBiosOverTCPIP = $false # Enables NetBIOS Over TCP/IP ($false = Default, $true = Enabled)
#-----------------------------------------------------------------------------------
Function ProcessVM {
	$vm = Get-VM $args[0]
	if (($vm | % {$_.PowerState}) -ne "PoweredOn") {Write-Host (get-date -uformat %I:%M:%S) "Bypassing shutdown virtual machine: $vm" -ForegroundColor Yellow;Return}
	if ((($vm | Get-View | % {$_.Config.GuestFullName}) -match "2003") -or (($vm | Get-View | % {$_.Config.GuestFullName}) -match "2008") -or (($vm | Get-View | % {$_.Config.GuestFullName}) -match "XP") -or (($vm | Get-View | % {$_.Config.GuestFullName}) -match "Vista") -or (($vm | Get-View | % {$_.Config.GuestFullName}) -match "7")){}else{Write-Host (get-date -uformat %I:%M:%S) "Bypassing legacy or non-Windows virtual machine: $vm" -ForegroundColor Red;Return}
	if (($vm | Get-View | % {$_.Guest.ToolsStatus}) -ne "toolsOk")  {Write-Host (get-date -uformat %I:%M:%S) "Bypassing virtual machine with outdated VMware Tools: $vm" -ForegroundColor RED;Return}
	if (($vm | Get-View | % {$_.Guest.ToolsRunningStatus}) -ne "guestToolsRunning") {Write-Host (get-date -uformat %I:%M:%S) "Bypassing virtual machine with VMware Tools not running: $vm" -ForegroundColor RED;Return}
	if(($null -eq (get-scsicontroller -vm $vm | Where {$_.Type -ne "ParaVirtual"})) -and ($null -eq (Get-NetworkAdapter -vm $vm | Where {$_.Type -ne "Vmxnet3"}))) {Write-Host (get-date -uformat %I:%M:%S) "Bypassing VM with VMxNet3 and Paravirtual Controller: $vm" -ForegroundColor Green;Return}
  Write-Host (get-date -uformat %I:%M:%S) "Processing virtual machine:" $vm -ForegroundColor Green
	$currentip = $vm.guest.ipaddress[0]
	$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "ipconfig /all > C:\ipconfig.log" -GuestCredential $vmcredentials
	if (!$?){Clear-host;Write-Host "Invalid credentials detected."`n"Please re-run the script and provide valid credentials." -ForegroundColor Green;Start-sleep -s 10;Exit}
	Copy-VMGuestFile -source "C:\ipconfig.log" -Destination "C:\ipconfig.log" -vm $vm -GuestCredential $vmcredentials -GuestToLocal:$true
	$ipconfig = Get-Content "C:\ipconfig.log"
	$subnetmask = $ipconfig | Where {$_ match Subnet Mask} | foreach-object{$_.Split(:)[1].Trim()}
	$gatewayaddress = $ipconfig match Default Gateway | foreach-object{$_.Split(:)[1].Trim()}
	$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "netsh interface ip show dnsservers > C:\DNS.log" -GuestCredential $vmcredentials
	Copy-VMGuestFile -source "C:\DNS.log" -Destination "C:\DNS.log" -vm $vm -GuestCredential $vmcredentials -GuestToLocal:$true
	$dnslog = get-content C:\DNS.log
	$dns = $dnslog -match "\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}" | % {$_.Trim()}
	$dns[0] = $dns[0].Split(":")[1].Trim()
	$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "netsh interface ip show winsservers > C:\WINS.log" -GuestCredential $vmcredentials
	Copy-VMGuestFile -source "C:\WINS.log" -Destination "C:\WINS.log" -vm $vm -GuestCredential $vmcredentials -GuestToLocal:$true
	$winslog = get-content C:\WINS.log
	$wins = $winslog -match "\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}" | % {$_.Trim()}
	if($wins.count -ne $null){$wins[0] = $wins[0].Split(":")[1].Trim()}
  Write-Host `t(get-date -uformat %I:%M:%S) "--Current virtual machine network settings--"`n`t"   IP: "$currentip" SM: "$subnetmask" DG: "$gatewayaddress`n`t"   DNS: "$dns -ForegroundColor Green
  if($wins.count -ne $null){Write-Host `t"   WINS: "$wins -ForegroundColor Green}
	$currentadapter = Get-NetworkAdapter -vm $vm
	$networkname = $currentadapter.NetworkName
  Write-Host `t(get-date -uformat %I:%M:%S) "Shutting down virtual machine" -ForegroundColor Green
	$invokeOut = shutdown-vmguest -vm $vm -Confirm:$false
	do {start-sleep -s 3;$powerstate = get-vm $vm | % {$_.PowerState}} while ($powerstate -ne "PoweredOff")
	if ((Get-VM $vm | Get-View | % {$_.Config.Version}) -eq "vmx-04") {Write-Host `t(get-date -uformat %I:%M:%S) "Upgrading to VM Hardware Version 7" -ForegroundColor Green;$invokeOut = Get-VM $vm | Get-View | % { $_.UpgradeVM_Task("vmx-07")};do {start-sleep -s 3;} while ((Get-VM $vm | Get-View | % {$_.Config.Version}) -ne "vmx-07")}
  Write-Host `t(get-date -uformat %I:%M:%S) "Removing legacy network adapter and adding VMxNet3 adapter" -ForegroundColor Green
	Get-NetworkAdapter -vm $vm | remove-networkadapter -confirm:$False
	$invokeOut = New-NetworkAdapter -vm $vm -Type vmxnet3 -NetworkName $networkname -StartConnected
  Write-Host `t(get-date -uformat %I:%M:%S) "Adding temporary hard drive on paravirtual SCSI controller" -ForegroundColor Green
	$invokeOut = $vm | New-HardDisk -CapacityKB 1024 -StorageFormat Thin | New-ScsiController -Type paravirtual
  Write-Host `t(get-date -uformat %I:%M:%S) "Starting virtual machine" -ForegroundColor Green
	$invokeOut = start-vm -vm $vm -Confirm:$false
	do {start-sleep -s 3;$vmwaretools = get-vm $vm | % {get-view $_.ID} | % {$_.Guest.ToolsRunningStatus}} while ($vmwaretools -eq "guestToolsNotRunning")
  Write-Host `t(get-date -uformat %I:%M:%S) "Running Devcon and RemoveHidden.bat to remove hidden devices" -ForegroundColor Green
	Copy-VMGuestFile -source devcon.exe -Destination "C:\" -vm $vm -GuestCredential $vmcredentials -GuestToLocal:$false
	Copy-VMGuestFile -source devcon64.exe -Destination "C:\" -vm $vm -GuestCredential $vmcredentials -GuestToLocal:$false
	Copy-VMGuestFile -source "C:\Windows\Temp\RemoveHidden.bat" -Destination "C:\" -vm $vm -GuestCredential $vmcredentials -GuestToLocal:$false
	$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "C:\removehidden.bat" -GuestCredential $vmcredentials 
	$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "C:\Devcon.exe rescan" -GuestCredential $vmcredentials 
	$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "C:\Devcon64.exe rescan" -GuestCredential $vmcredentials 
  Write-Host `t(get-date -uformat %I:%M:%S) "Renaming Network Connections to Local Area Connection" -ForegroundColor Green
	for ($i = 2; $i -lt 10; $i++) {$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "Netsh.exe interface set interface name=""Local Area Connection $i"" newname=""Local Area Connection""" -GuestCredential $vmcredentials}
  Write-Host `t(get-date -uformat %I:%M:%S) "Assigning IP, SM, and DG to Local Area Connection" -ForegroundColor Green
	$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "Netsh.exe interface ip set address ""Local Area Connection"" static $currentip $subnetmask $gatewayaddress 1" -GuestCredential $vmcredentials
  Write-Host `t(get-date -uformat %I:%M:%S) "Assigning DNS Server addresses to Local Area Connection" -ForegroundColor Green
	for ($i = 0; $i -lt $dns.count; $i++){$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText ("Netsh.exe interface ip add dns ""Local Area Connection"" " + $dns[$i] + " index=" + ($i+1)) -GuestCredential $vmcredentials}
 if($wins.count -ne $null){
  Write-Host `t(get-date -uformat %I:%M:%S) "Assigning WINS Server addresses to Local Area Connection" -ForegroundColor Green
	for ($i = 0; $i -lt $wins.count; $i++){$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText ("Netsh.exe interface ip add wins ""Local Area Connection"" " + $wins[$i] + " index=" + ($i+1)) -GuestCredential $vmcredentials}}
	Copy-VMGuestFile -source nvspbind.exe -Destination "C:\" -vm $vm -GuestCredential $vmcredentials -GuestToLocal:$false
	if ($DisablePacer -eq $true) {Write-Host `t(get-date -uformat %I:%M:%S) "Removing QoS Packet Scheduler" -ForegroundColor Green;$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "C:\nvspbind.exe /d ""Local Area Connection"" ms_pacer" -GuestCredential $vmcredentials}
	if ($DisableIPv6 -eq $true) {Write-Host `t(get-date -uformat %I:%M:%S) "Disabling IPv6" -ForegroundColor Green;$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "C:\nvspbind.exe /d ""Local Area Connection"" ms_tcpip6" -GuestCredential $vmcredentials}
	if ($DisableLLTDIO -eq $true) {Write-Host `t(get-date -uformat %I:%M:%S) "Removing Link-Layer Topology Discovery Mapper" -ForegroundColor Green;$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "C:\nvspbind.exe /d ""Local Area Connection"" ms_lltdio" -GuestCredential $vmcredentials}
	if ($DisableRSPNDR -eq $true) {Write-Host `t(get-date -uformat %I:%M:%S) "Removing Link-Layer Topology Discovery Responder" -ForegroundColor Green;$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "C:\nvspbind.exe /d ""Local Area Connection"" ms_rspndr" -GuestCredential $vmcredentials}
	if ($EnableNetBiosOverTCPIP -eq $true) {Write-Host `t(get-date -uformat %I:%M:%S) "Enabling NetBIOS over TCP/IP" -ForegroundColor Green;$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "wmic /interactive:off nicconfig where TcpipNetbiosOptions=0 call SetTcpipNetbios 1" -GuestCredential $vmcredentials}
  Write-Host `t(get-date -uformat %I:%M:%S) "Shutting down virtual machine" -ForegroundColor Green
	$invokeOut = shutdown-vmguest -vm $vm -Confirm:$false
	do {start-sleep -s 3;$powerstate = get-vm $vm | % {$_.PowerState}} while ($powerstate -ne "PoweredOff")
  Write-Host `t(get-date -uformat %I:%M:%S) "Removing temporary hard drive from virtual machine" -ForegroundColor Green
	Get-HardDisk -vm $vm | Where {$_.CapacityKB -eq 1024} | Remove-HardDisk -DeletePermanently -Confirm:$False
  Write-Host `t(get-date -uformat %I:%M:%S) "Changing Primary SCSI controller to paravirtual" -ForegroundColor Green
	$invokeOut = Get-HardDisk -VM $vm | Select -First 1 | Get-ScsiController | Set-ScsiController -Type paravirtual
  Write-Host `t(get-date -uformat %I:%M:%S) "Starting virtual machine" -ForegroundColor Green
	$invokeOut = start-vm -vm $vm -Confirm:$false
	do {start-sleep -s 3;$vmwaretools = get-vm $vm | % {get-view $_.ID} | % {$_.Guest.ToolsRunningStatus}} while ($vmwaretools -eq "guestToolsNotRunning")
  Write-Host `t(get-date -uformat %I:%M:%S) "Performing virtual machine temporary file cleanup" -ForegroundColor Green
	$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "del C:\ipconfig.log /f" -GuestCredential $vmcredentials
	$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "del C:\DNS.log /f" -GuestCredential $vmcredentials
	$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "del C:\WINS.log /f" -GuestCredential $vmcredentials
	$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "del C:\devcon.exe /f" -GuestCredential $vmcredentials
	$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "del C:\devcon64.exe /f" -GuestCredential $vmcredentials
	$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "del C:\RemoveHidden.bat /f" -GuestCredential $vmcredentials
	$invokeOut = Invoke-VMScript -vm $vm -ScriptType bat -ScriptText "del C:\nvspbind.exe /f" -GuestCredential $vmcredentials
	$invokeOut = del C:\ipconfig.log
	$invokeOut = del C:\DNS.log
	$invokeOut = del C:\WINS.log
}
#-----------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------
# Begin function to Create C:\Windows\temp\RemoveHidden.bat file for removing non-present devices from virtual machines
#-----------------------------------------------------------------------------------
Function CreateRemoveHiddenBatch {
	set-content "C:\Windows\Temp\RemoveHidden.bat" "@Echo Off"
	add-content "C:\Windows\Temp\RemoveHidden.bat" "C:\DEVCON Find * | FIND /I /V `"matching device(s)`" > `"%Temp%\DevconFind.txt`""
	add-content "C:\Windows\Temp\RemoveHidden.bat" "IF NOT EXIST `"%Temp%\DevconFind.txt`" GOTO Syntax"
	add-content "C:\Windows\Temp\RemoveHidden.bat" "FOR /F `"tokens=1 delims=: `" %%A IN ('C:\DEVCON FindAll * ^| FIND /I /V `"matching device(s)`"') DO ("
	add-content "C:\Windows\Temp\RemoveHidden.bat" "TYPE `"%Temp%\DevconFind.txt`" | FIND `"%%~A`" >NUL"
	add-content "C:\Windows\Temp\RemoveHidden.bat" "IF ERRORLEVEL 1 %Debug% C:\DEVCON Remove `"@%%~A`""
	add-content "C:\Windows\Temp\RemoveHidden.bat" ")"
	add-content "C:\Windows\Temp\RemoveHidden.bat" "DEL `"%Temp%\DevconFind.txt`""
	add-content ""
	add-content "C:\Windows\Temp\RemoveHidden.bat" "C:\DEVCON64.EXE Find * | FIND /I /V `"matching device(s)`" > `"%Temp%\DevconFind.txt`""
	add-content "C:\Windows\Temp\RemoveHidden.bat" "IF NOT EXIST `"%Temp%\DevconFind.txt`" GOTO Syntax"
	add-content "C:\Windows\Temp\RemoveHidden.bat" "FOR /F `"tokens=1 delims=: `" %%A IN ('C:\DEVCON64.EXE FindAll * ^| FIND /I /V `"matching device(s)`"') DO ("
	add-content "C:\Windows\Temp\RemoveHidden.bat" "TYPE `"%Temp%\DevconFind.txt`" | FIND `"%%~A`" >NUL"
	add-content "C:\Windows\Temp\RemoveHidden.bat" "IF ERRORLEVEL 1 %Debug% C:\DEVCON64.EXE Remove `"@%%~A`""
	add-content "C:\Windows\Temp\RemoveHidden.bat" ")"
	add-content "C:\Windows\Temp\RemoveHidden.bat" "DEL `"%Temp%\DevconFind.txt`""
	Write-Host (get-date -uformat %I:%M:%S) "RemoveHidden.bat created successfully." -ForegroundColor Green
}
#-----------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------
# Begin processing script with user input
#-----------------------------------------------------------------------------------
if ($env:Processor_Architecture -ne "x86"){&"$env:windir\syswow64\windowspowershell\v1.0\powershell.exe" -noninteractive -noprofile -file $myinvocation.Mycommand.path -executionpolicy bypass;exit}
if (![System.IO.File]::Exists("devcon.exe")){Write-Host "Unable to locate Devcon.exe in"([System.IO.Directory]::GetCurrentDirectory())"`nPlease copy Devcon.exe to the current directory then relaunch to proceed.`nFor more information visit http://support.microsoft.com/kb/311272" -ForegroundColor Red;start-sleep -s 30;Exit}
if (![System.IO.File]::Exists("nvspbind.exe")){Write-Host "Unable to locate nvspbind.exe in"([System.IO.Directory]::GetCurrentDirectory())"`nPlease copy nvspbind.exe to the current directory then relaunch to proceed.`nTo download, please visit http://archive.msdn.microsoft.com/nvspbind" -ForegroundColor Red;start-sleep -s 30;Exit}
Add-PSSnapin VMware.VimAutomation.Core
[void][System.Reflection.Assembly]::LoadWithPartialName('Microsoft.VisualBasic')
$vmlist = [Microsoft.VisualBasic.Interaction]::InputBox("Provide a list of all virtual machines:" + [Environment]::NewLine + [Environment]::NewLine + "Enter * for all virtual machines in vCenter, or" + [Environment]::NewLine + "Enter a comma seperated list of virtual machine names.", "Virtual Machine List", "*")
if ($vmlist -eq "") {Write-Host "Cancel detected, please execute again to proceed." -ForegroundColor Red;Start-Sleep -S 30} elseif ($vmlist -eq "*"){
	$vbscript = New-Object -ComObject wscript.shell
	$confirm = $vbscript.popup("Proceed with processing all virtual machines in vCenter?",0,"Proceed?",4)
	if ($confirm -ne 6){Write-Host "Cancel detected, please execute again to proceed." -ForegroundColor Red;Start-Sleep -S 30} elseif ($confirm -eq 6){
		if (![System.IO.File]::Exists("C:\Windows\Temp\RemoveHidden.bat")){CreateRemoveHiddenBatch}
		connect-viserver localhost
		while ($vmcredentials -eq $null) {Clear-host;Write-Host "Please provide authentication credentials for VM Guest Operating System(s)" -ForegroundColor Green;$vmcredentials = Get-Credential -Credential ""}
		$vms = Get-VM
		foreach ($vmname in $vms){
			ProcessVM $vmname.Name
		}
		Write-Host (get-date -uformat %I:%M:%S) "Processing completed! Please review change log above." -ForegroundColor Green
		Start-Sleep -s 30
	}
} else {
	foreach ($vmname in $vmlist.Split(",")){
		if (![System.IO.File]::Exists("C:\Windows\Temp\RemoveHidden.bat")){CreateRemoveHiddenBatch}
		connect-viserver localhost
		while ($vmcredentials -eq $null) {Clear-host;Write-Host "Please provide authentication credentials for VM Guest Operating System(s)" -ForegroundColor Green;$vmcredentials = Get-Credential -Credential ""}
		ProcessVM $vmname
	}
	Write-Host (get-date -uformat %I:%M:%S) "Processing completed! Please review change log above." -ForegroundColor Green
	Start-Sleep -s 30
}
#-----------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------
#Automated Migration to VMxNet3 and Paravirtual Controller for vSphere 4.x
#Created by Dane Young, Entisys Solutions 2011
#Build 2011.5.15 Revision 3
#-----------------------------------------------------------------------------------